package {
	import __AS3__.vec.Vector;
	
	import flash.display.*;
	import flash.events.*;
	import flash.geom.*;

	[SWF(width=250, height=250)]
	public class phong extends Sprite
	{
		private var canvas:Sprite;
		private var sphere:Sphere;
		private var lightMap:BitmapData;
		private var projMtx:Matrix3D;
		
		public function phong()
		{
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.quality = StageQuality.BEST;

			sphere = new Sphere(30);

			var lightVec:Vector3D = new Vector3D(0,1,0);
			var eyeVec:Vector3D = new Vector3D(0,0,-1);
			var half:Vector3D = lightVec.add(eyeVec);
			half.normalize();
			
			var halfMat:Matrix3D = lookAtLH(half, new Vector3D(0,0,0), new Vector3D(0,1,0));
			
			sphere.calcUV(halfMat);
			
			lightMap = this.makeLightMap(0xffffff, 0x990000, 0x111111, 220);
			canvas = new Sprite();
			addChild(canvas);

			var fov:Number = 3.14/180 * 40;
			var aspect:Number = stage.stageWidth/stage.stageHeight;
			var nearClip:Number = 0.001;
			var farClip:Number = 500;
			projMtx = perspective(fov, aspect, nearClip, farClip);

			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			
		}
		
		private function onEnterFrame(event:Event):void
		{
			canvas.x = stage.stageWidth/2;
			canvas.y = stage.stageHeight/2;

			phongShading();
		}
		
		private function phongShading():void
		{
			var pos:Vector3D = new Vector3D(0,0,1);
			var target:Vector3D = new Vector3D(0,0,0);
			var up:Vector3D = new Vector3D(0,1,0);

			var wMtx:Matrix3D = lookAtLH(pos, target, up);
			wMtx.appendTranslation(0,0,20);
			wMtx.append(projMtx);
			wMtx.appendScale(stage.stageWidth, stage.stageHeight, 1);
						
			// transform
			Utils3D.projectVectors(wMtx, sphere.vertices, sphere.projected_vertices, sphere.uvts);
			canvas.graphics.clear();
			canvas.graphics.beginBitmapFill(lightMap);
			canvas.graphics.drawTriangles(sphere.projected_vertices, sphere.indices, sphere.uvts, TriangleCulling.POSITIVE);
			canvas.graphics.endFill();			
		}
		
		private function lookAtLH(pos:Vector3D, target:Vector3D, up:Vector3D):Matrix3D
		{
			var z:Vector3D = pos.subtract(target);
			z.normalize();
			var x:Vector3D = up.crossProduct(z);
			x.normalize();
			var y:Vector3D = z.crossProduct(x);
			
			var viewMat:Matrix3D = new Matrix3D();
			viewMat.rawData = Vector.<Number>([
				x.x, y.x, z.x, 0,
				x.y, y.y, z.y, 0,
				x.z, y.z, z.z, 0,
				-x.dotProduct(pos),-y.dotProduct(pos),-z.dotProduct(pos),1
			]);
			return viewMat;
		}
		
		private function perspective(fov:Number, aspect:Number, zNear:Number, zFar:Number):Matrix3D
		{
			var h:Number = 1.0/Math.tan(fov/2.0);
			var w:Number = h / aspect;
			
			var mat:Matrix3D = new Matrix3D();
			var data:Vector.<Number> = Vector.<Number> ([
				w,0,0,0,
				0,h,0,0,
				0,0,zFar/(zFar-zNear),1,
				0,0,(-zNear*zFar/(zFar-zNear)),0
			]); 

			mat.rawData = data;
			
			return mat;
		}
		
		public function makeLightMap(lightColor:int, diffuseColor:int, ambientColor:int, specularLevel:int):BitmapData
		{
			var w:Number = 1024;
			var mat:Matrix = new Matrix();
			var s:Sprite = new Sprite();
			mat.createGradientBox(w,w,0,0,0);
			
			s.graphics.beginGradientFill(GradientType.RADIAL, [lightColor,diffuseColor,ambientColor],
				 [1,1,1], [0,255-specularLevel, 255], mat);
			s.graphics.drawRect(0,0,w,w);
			s.graphics.endFill();
			
			var bmp:BitmapData = new BitmapData(w,w,false,0x0000ff);
			bmp.draw(s);

			return bmp;			
		}
	}
}
